'' =================================================================================================
''
''   File....... jm_buttons_latch.spin
''   Purpose....
''   Author..... Jon "JonnyMac" McPhalen
''               Copyright (C) 2015 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com
''   Started....
''   Updated....
''
'' =================================================================================================


con { timing }

  _clkmode = xtal1 + pll16x                                     
  _xinfreq = 5_000_000                                          ' use 5MHz crystal

  CLK_FREQ = (_clkmode >> 6) * _xinfreq                         ' system freq as a constant
  MS_001   = CLK_FREQ / 1_000                                   ' ticks in 1ms
  US_001   = CLK_FREQ / 1_000_000                               ' ticks in 1us


con { io pins }

  RX1  = 31                                                     ' programming / terminal
  TX1  = 30

  SDA  = 29                                                     ' eeprom / i2c
  SCL  = 28

  MOD0 = 19
  MOD9 = 18
  MOD8 = 17 
  MOD7 = 16 
  MOD6 = 15 
  MOD5 = 14 
  MOD4 = 13
  MOD3 = 12
  MOD2 = 11
  MOD1 = 10

  BTN0 =  9
  BTN9 =  8
  BTN8 =  7 
  BTN7 =  6 
  BTN6 =  5 
  BTN5 =  4 
  BTN4 =  3
  BTN3 =  2
  BTN2 =  1
  BTN1 =  0


obj

' main                                                          ' * master Spin cog
  time : "jm_time"                                              '   timing and delays
  prng : "jm_prng"                                              '   pseudo-random number generator
  io   : "jm_io_basic"                                          '   essential io
  term : "jm_fullduplexserial"                                  ' * serial IO for terminal
                                                                 
' * uses cog when loaded                                         
                                                                 
                                                                 
var

  long  nbtns                                                   ' new buttons
  long  obtns                                                   ' old buttons
  long  xbtns                                                   ' newly pressed buttons

  long  relays                                                          
                                                                 
                                                                 
pub main | modebits, btn, mask                                                      
                                                                 
  setup

  repeat
    scan_buttons                                                ' scan the buttons
    modebits := ina[MOD0..MOD1]                                 ' scan the mode inputs

    repeat btn from 0 to 9
      mask := 1 << btn                                          ' create mask for button
      ifnot (modebits & mask)                                   ' momentary mode?
        if (nbtns & mask)                                       ' button pressed?
          relays |= mask                                        '  relay on
        else                                                    ' else
          relays &= !mask                                       '  relay off

      else                                                      ' latch mode
        if (xbtns & mask)                                       '  new press?
          relays ^= mask                                        '   toggle relay
      
    ' set relays
    ' play random tone


pub scan_buttons | t

  obtns := nbtns                                                ' save last

  nbtns := %11_11111111                                         ' arm all inputs

  t := cnt
  repeat 25                                                     ' 25ms debounce
    waitcnt(t += MS_001)                                        ' wait 1ms
    nbtns &= ina                                                ' (re)scan all inputs

  xbtns := nbtns ^ obtns & nbtns                                ' new presses (0 -> 1)

                                                                 
pub setup                                                        
                                                                 
'' Setup IO and objects for application                          
                                                                 
  time.start                                                    ' setup timing & delays
                                                                 
  prng.seed(-cnt, -cnt ~> 2, $EA7_BEEF, cnt << 2, cnt)          ' seed randomizer
                                                                 
  io.start(0, 0)                                                ' clear all pins (master cog)
                                                                 
  term.start(RX1, TX1, %0000, 115_200)                          ' start serial for terminal
                                                                 
                    
dat { license }                                                  

{{

  Copyright (C) 2016 Jon McPhalen

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}